`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 		 MIG Internacional
// Engineer: 		 Juan Jos Rondn
// 
// Create Date:    11:32:24 07/17/2008 
// Design Name: 
// Module Name:    writedata 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module writedata(ctrbus,			//Enable signal
					ctraddr,				//Output control for address bus
					ctrdata,				//Output control for data bus
					donedato,			//Write data in SDRAM is done
					addrout,				//Output register for address bus
					sys_busen,			//Enable signal for SDRAM
					sys_sdcas,			//CAS signal for SDRAM
					sys_sdras,			//RAS signal for SDRAM
					sys_sdwe,			//WE signal for SDRAM
					ba0,					//Bank select 0 for SDRAM
					ba1,					//Bank select 1 for SDRAM
					writeok,				//Done write SDRAM indicator (for Debugg)
					dirram,				//Initial address for SDRAM write
					clkdiv,				//Clock signal (50 MHz)
					dataout);			//Output register for data bus

input clkdiv,ctrbus;
input [31:0] dirram;
output donedato,sys_busen,sys_sdcas,sys_sdras,sys_sdwe,ba0,ba1,writeok,ctraddr,ctrdata;
output [12:0] addrout;
output [31:0] dataout;

//States definition

parameter [2:0] init=0;
parameter [2:0] prechargewr=1;
parameter [2:0] activaterd=2;
parameter [2:0] write1=3;
parameter [2:0] finishwr=4;

reg sys_busen=1;
reg sys_sdras=1'bz;
reg sys_sdcas=1'bz;
reg sys_sdwe=1'bz;
reg donedato=0;
reg [12:0] addrout=0;
reg ctraddr=0;
reg ctrdata=0;
reg [2:0] wrstate=init; //State machine
reg writeok=0;
reg [31:0] dataout = 32'h00000000;
reg [17:0] countdata=0;	//Data write count
reg [12:0] fila=0;		//Row Address for SDRAM
reg [12:0] columna=0;	//Column Address for SDRAM
reg [1:0] banco=2'bz;	//Bank select

assign ba0=banco[0];
assign ba1=banco[1];
	
always @ (negedge clkdiv) begin
	case (wrstate)
		init: begin	
			if (ctrbus) begin						//Enable signal is active
				writeok <= 0;		
				donedato <= 0;
				sys_busen <= 1;
				sys_sdcas <= 1;
				sys_sdras <= 1;
				sys_sdwe <= 1;
				fila <= dirram[23:11];			//Read the initial row address
				columna [12:9] <= 4'h0;
				columna[8:0] <= dirram[10:2]; //Read the initial column address
				banco <= dirram[25:24]; 		//Read the initial bank
				wrstate <= prechargewr;
				countdata <= 0;
			end
			else begin
				donedato <= 0;
				ctraddr <=0;
				ctrdata <= 0;
				sys_busen <= 1;
			end
		end
		prechargewr: begin						//Send precharge command 
				sys_busen <= 0;
				ctrdata <= 0;
				ctraddr <= 1;						//Enable write in address bus
				addrout[12:0] <= fila;			//Write fila address
				sys_sdcas <= 1;
				sys_sdras <= 0;
				sys_sdwe <= 0;
				wrstate <= activaterd;
		end
		activaterd: begin							//Send activate row
			sys_sdwe <= 1;
				wrstate <= write1;
		end
		write1: begin								//Write data in SDRAM
			sys_sdras <= 1;
			sys_sdcas <= 0;
			addrout [12:0] <= columna;			//Write column address
			sys_sdwe <= 0;
			ctrdata <= 1;
			dataout <= countdata;				//Write countdata in SDRAM
			if (countdata == 76799) begin 	//End of data array
				wrstate <= finishwr;
				countdata <= 0;
			end
			else begin
				countdata <= countdata + 1;	//Increment countdata
				if (columna == 511) begin		//End of column
					if (fila == 8191) begin		//End of row
						fila <= 0;
						columna <= 0;
						banco <= banco + 1;		//change bank
						wrstate <= prechargewr; //go to precharge
					end
					else begin
						columna <= 0;
						fila <= fila +1;			//change row
						wrstate <= prechargewr;	//go to precharge
					end
				end
				else begin
					columna <= columna + 1;
				end
			end				
		end
		finishwr: begin
			writeok <= 1;							//finish write indicator to debugg
			sys_sdras <= 1;
			sys_sdcas <= 1;
			sys_sdwe <= 1;
			donedato <=1;							//finish write indicator to trackwrite module
			ctraddr <= 0;
			ctrdata <= 0;
			sys_busen <= 1;
			wrstate <= init;						//Begin again
		end
		default:
			wrstate <= init;
	endcase
end			

endmodule
